/*
** Restricted Rights Legend
**
** Use, duplication or disclosure by the government is subject to restrictions
** as set forth in subparagraph (c)(1)(ii) of the Rights in Technical Data and
** Computer Software clause at DFARS 252.227-7013.
**
** Copyright (c) 1997 Lockheed Martin C2 Integration Systems
**
**
** File:         KqmlKeywords.java
**
** Description:  Contains the KqmlKeywords class that defines KQML
**               protocol field names and standard performatives.
**
*/

package kqml;

/**
 * A class that defines the constant names of KQML protocol fields
 * and standard performatives that can be used in an "implements"
 * clause of a class definition.
 *
 * @author Peter Stevens
 * @version $Revision: 1.3 $
 * @see KqmlAgent
 * @see Message
 * @see Reply
 */

public interface KqmlKeywords
{
    /**
     * KQML Protocol level field names
     */
    public  static final String CONTENT      = ":content";
    public  static final String IN_REPLY_TO  = ":in-reply-to";
    public  static final String KQML_MSG_ID  = ":kqml-msg-id";
    public  static final String KRIL_WRAPPER = ":kril-wrapper";
    public  static final String ONTOLOGY     = ":ontology";
    public  static final String RECEIVER     = ":receiver";
    public  static final String REPLY_WITH   = ":reply-with";
    public  static final String SENDER       = ":sender";

    /**
     * KQML Performatives
     *
     * Common values of response performatives.
     */
    public  static final String ERROR        = "error";
    public  static final String REPLY        = "reply";
    public  static final String SORRY        = "sorry";

    /**
     * Query performatives
     */
    public  static final String ASK_ALL      = "ask-all";
    public  static final String ASK_IF       = "ask-if";
    public  static final String ASK_IN       = "ask-in";
    public  static final String ASK_ONE      = "ask-one";

    /**
     * Multi-response Query performatives
     */
    public  static final String STREAM_ALL   = "stream-all";
    public  static final String STREAM_IN    = "stream-in";

    /**
     * Generic Informational performatives
     */
    public  static final String ACHIEVE      = "achieve";
    public  static final String CANCEL       = "cancel";
    public  static final String TELL         = "tell";
    public  static final String UNACHIEVE    = "unachieve";
    public  static final String UNTELL       = "untell";

    /**
     * Generator performatives
     */
    public  static final String DISCARD      = "discard";
    public  static final String GENERATOR    = "generator";
    public  static final String NEXT         = "next";
    public  static final String READY        = "ready";
    public  static final String REST         = "rest";
    public  static final String STANDBY      = "standby";

    /**
     * Capbility Definition performatives
     */
    public  static final String ADVERTISE    = "advertise";
    public  static final String EXPORT       = "export";
    public  static final String IMPORT       = "import";
    public  static final String MONITOR      = "monitor";
    public  static final String SUBSCRIBE    = "subscribe";

    /**
     * Networking performatives
     */
    public  static final String BROADCAST    = "broadcast";
    public  static final String FORWARD      = "forward";
    public  static final String REGISTER     = "register";
    public  static final String ROUTE        = "route";
    public  static final String UNREGISTER   = "unregister";
}
