/* File: MainFrame.java
 * Contains: ACP3 COA Matrix Tests
 * Author: Austin Tate <a.tate@ed.ac.uk>
 * Created: Wed Jan  7 15:14:27 1998
 * Updated: Wed Jan 21 19:09:53 1998 by Jeff Dalton
 * Copyright: (c) 1998, AIAI, University of Edinburgh
 */

/** ACP3 - a sample java application
    @version 0.1
    @author Austin Tate
 */

/* Conventions:
   package, thispack (short name, no uppercase)
   ClassName, ThisIsAClass
   methodName, thisIsAMethod()
   variableName, thisIsAVariable
   CONSTANT, THIS_IS_A_CONSTANT
 */

package oplan.acp3;

import java.awt.*;
import java.awt.image.*;
import java.net.*;
import java.util.*;
import oplan.util.*;

class LogoCanvas extends Canvas {
 
   private int canvasWidth;
   private int canvasHeight;
   private Dimension myDimension;
   private Image logo;   
   private Image offsc;
   private Graphics osg;
      
   LogoCanvas(int x, int y, Image logo) {
      super();
      canvasWidth=x;
      canvasHeight=y;
      myDimension=new Dimension(x,y);
      this.logo=logo;
   }
 
   public Dimension minimumSize() {
          return myDimension;
   }
 
   public Dimension preferredSize() {
          return myDimension;
   }
 
   public void update (Graphics g) {
       Debug.noteln("LogoCanvas update called");
       paint(g);
   }
   
   public void paint(Graphics g) {  
       if (offsc==null) {
           offsc=createImage(canvasWidth,canvasHeight);
           osg=offsc.getGraphics();
           // once and for all setup of logo, etc into off screen image
           // image is definitiely available - so size known
           osg.setColor(Color.gray);
           osg.drawLine(10,5,(this.getSize().width)-10,5);
           osg.setFont(DefaultFont.LARGE);
           osg.drawString("ACP",20,32+5);
           osg.setFont(DefaultFont.LARGE_SUPERSCRIPT);
           osg.drawString("3",92,20);
           osg.setFont(DefaultFont.NORMAL);
           osg.drawString("Based on O-Plan Technology",110,22);
           osg.drawString("Funded by ARPI",110,36);
           osg.drawImage(logo,
                (this.getSize().width)-(logo.getWidth(this)+20),10,this);
       }
       g.drawImage(offsc,0,0,this);
   }

}

class MainFrame extends Frame {
    ViewMatrix vm;
    ViewGraph vg;
    InfoFrame info;
    
    public Node m_hitNode = null;  // *** added by JL ***

    private Dimension myDimension;
    
    boolean StartCOA2 = false;
    boolean StartCOA3 = false;
    int ScreenNumber = -1;

    private static final String mfLabel="ACP3";
    private static final String infoLabel="ACP3 Information";
    private static final int mfWidth=600, mfHeight=700, mfXPosn=50, mfYPosn=50,
                     infoWidth=350, infoHeight=400, infoXPosn=660, infoYPosn=50;
   
    MainFrame() {
        super(mfLabel);

        // set up instance variable values
        myDimension=new Dimension(mfWidth,mfHeight);                

        //initialise InfoFrame and put up splash screen
        info=new InfoFrame(infoLabel,infoWidth,infoHeight);
        info.setBackground(Color.white);
        info.resize(mfWidth,mfHeight);
        info.setLocation(infoXPosn,infoYPosn);
        info.pack();
        info.show();
 
        //initialise MainFrame
        setBackground(Color.white);
        setFont(DefaultFont.NORMAL);
        resize(mfWidth,mfHeight);
        setLocation(mfXPosn,mfYPosn);
 
        // setup MainFrame Layout
        GridBagLayout mfLayout=new GridBagLayout();
        GridBagConstraints gblConstraints=new GridBagConstraints();
        setLayout(mfLayout);
                
        int componentHeight; //temp variable used to hold component height
        double componentRatio; //total of all MainFrame component ratios=1.0

        gblConstraints.fill=GridBagConstraints.BOTH; //components fill their allocated areas
        gblConstraints.weightx=1.0; //stretch out to max. width (for one column layout)

        // Menubar
        MenuBar mfMenuBar=new MenuBar();
        Menu mfMenuFile=new Menu("File");
        Menu mfMenuWindows=new Menu("Windows");
	Menu mfHelpMenu=new Menu("Help");
        MenuItem mfMenuFileReset=new MenuItem("Reset");
        MenuItem mfMenuFileRefresh=new MenuItem("Refresh");
        MenuItem mfMenuFileQuit=new MenuItem("Quit");
	MenuItem mfHelpMenuIntro=new MenuItem("Intro");
	MenuItem mfHelpMenuHelp=new MenuItem("Help");
	MenuItem mfHelpMenuKey=new MenuItem("Key");
        mfMenuFile.add(mfMenuFileReset);
        mfMenuFile.add(mfMenuFileQuit);
        mfMenuWindows.add(mfMenuFileRefresh);
	mfHelpMenu.add(mfHelpMenuIntro);
	mfHelpMenu.add(mfHelpMenuHelp);
	mfHelpMenu.add(mfHelpMenuKey);
        mfMenuBar.add(mfMenuFile);
        mfMenuBar.add(mfMenuWindows);
	mfMenuBar.add(mfHelpMenu);
        mfMenuBar.setHelpMenu(mfHelpMenu);
	setMenuBar(mfMenuBar);

        // Panel 1 - ViewGraph in a ScrollPane
        ScrollPane vgsp=new ScrollPane();        
        componentRatio=0.25;
        componentHeight=(int)(mfHeight*componentRatio); 
        gblConstraints.gridx=0;
        gblConstraints.gridy=0;
        gblConstraints.weighty=componentRatio;
        mfLayout.setConstraints(vgsp,gblConstraints);
        add(vgsp);        
        //now add ViewGraph to scollpane
        vg=new ViewGraph(this);  // *** changed by JL from () to (this) ***

        vgsp.add(vg);
        
        // Panel 2 - for buttons
        Panel panel2=new Panel();
        componentRatio=0.05;
        componentHeight=(int)(mfHeight*componentRatio); 
        gblConstraints.gridx=0;
        gblConstraints.gridy=1;
        gblConstraints.weighty=componentRatio;
        mfLayout.setConstraints(panel2,gblConstraints);
        panel2.add(new Button("Intro"));
        panel2.add(new Button("Help"));
        panel2.add(new Button("Key"));
        panel2.add(new Button("Hide Info"));
        panel2.add(new Button("Reset"));
        panel2.add(new Button("0"));
        panel2.add(new Button("1"));
        panel2.add(new Button("2"));
        panel2.add(new Button("3"));
        panel2.add(new Button("4"));  // *** added by JL ***
        panel2.add(new Button("5"));  // *** added by JL ***
        panel2.add(new Button("6"));  // *** added by JL ***
        panel2.add(new Button("Quit"));
        add(panel2);
        
        // Panel 3 - ViewMatrix in a ScrollPane
        ScrollPane vmsp=new ScrollPane(); 
        componentRatio=0.625; 
        componentHeight=(int)(mfHeight*componentRatio);
        gblConstraints.gridx=0;
        gblConstraints.gridy=2;
        gblConstraints.weighty=componentRatio;
        mfLayout.setConstraints(vmsp,gblConstraints);        
        add(vmsp);
        //now add ViewMatrix to scollpane
        vm=new ViewMatrix(this);
        //note mainfarme passed to ViewMatrix for container validation
        vmsp.add(vm);
        
        // Panel 4
        componentRatio=0.075;
        componentHeight=(int)(mfHeight*componentRatio);
        LogoCanvas logoCanvas=new LogoCanvas(mfWidth,componentHeight,
                      WaitImage.waitGetImage("IMG/aiai-logo-small.gif",this));                                     
        gblConstraints.gridx=0;
        gblConstraints.gridy=3;
        gblConstraints.weighty=componentRatio;
        mfLayout.setConstraints(logoCanvas,gblConstraints);
        add(logoCanvas);
        
        // Now show MainFrame
        pack();
        show();
	Debug.noteln("vg.size = " + vg.size());
	vg.start();  // *** added by JL ***
	vm.start();  // *** added by JL ***
    }
 
    public Dimension minimumSize() {
        return myDimension;
    }
 
    public Dimension preferredSize() {
        return myDimension;
    }
 
/*  public boolean not_handleEvent(Event e) {  // *** added by JL ***
      switch (e.id) { 
      case Event.ACTION_EVENT:
	return action(e,e.arg);
      case Event.WINDOW_MOVED:
	repaint();
	vg.repaint();
	break;
      case Event.WINDOW_EXPOSE:
	repaint();
	vg.repaint();
	break;
      case Event.WINDOW_DEICONIFY:
	repaint();
	vg.repaint();
	break;
      case Event.WINDOW_DESTROY:
	this.destroy();
	break;
      case Event.MOUSE_MOVE:
	break;
      default:
	repaint();
	vg.repaint();
	return false;
      }
      return true;
    } */

    public boolean handleEvent(Event ev) {
        if (ev.id==Event.WINDOW_DESTROY) {
           this.dispose();
           this.destroy(); //only if MainFrame is treated as primary interface?
           return true;
        }
        return super.handleEvent(ev);
    }
    
    public boolean action(Event ev, Object arg) {
        if ((ev.target instanceof MenuItem) || (ev.target instanceof Button)) {
            if (arg.equals("Quit")) {
                this.destroy();
                return true;
            }
            if (arg.equals("Refresh")) {
		repaint();
	        info.repaint();
		vg.repaint();
		vm.repaint();
		return true;
            }
            if (arg.equals("Reset")) {
	        vg.start();
		vm.vmClear();
                info.setInfo(info.SPLASH);                     
                info.repaint(); // will set InfoFrame as top frame
		ScreenNumber = -1;
                return true;
            }
            if (arg.equals("Intro")) {
                info.setInfo(info.INTRODUCTION);                     
                info.repaint(); // will set InfoFrame as top frame
                return true;
            }
            if (arg.equals("Help")) {
                info.setInfo(info.HELP);                     
                info.repaint();              
                return true;
            }
            if (arg.equals("Key")) {
                info.setInfo(info.COLOR_KEY);                     
                info.repaint();              
                return true;
            }
	    // *** start of section modified by JL ***
            if (arg.equals("0")) {
                //sample only
                vm.paintMatrix(0);
		vg.paintGraph(0);
		info.setInfo(info.DEMO_0);
		ScreenNumber = 0;
                return true;
            }
            if (arg.equals("1")) {
                //sample only
                vm.paintMatrix(1);
		vg.paintGraph(1);
		info.setInfo(info.DEMO_1);
		ScreenNumber = 1;
                return true;
            }
            if (arg.equals("2")) {
                //sample only
                vm.paintMatrix(2);
		vg.paintGraph(2);  
		info.setInfo(info.DEMO_2);
		ScreenNumber = 2;
                return true;
            }
            if (arg.equals("3")) {
                //sample only
                vm.paintMatrix(3);
		vg.paintGraph(3);  
		info.setInfo(info.DEMO_3);
		ScreenNumber = 3;
                return true;
            }
            if (arg.equals("3a")) {
                //sample only
	        startAction("develop COA",2);
                return true;
            }
            if (arg.equals("3b")) {
                //sample only
	        startAction("develop COA",3);
                return true;
            }
            if (arg.equals("4")) { 
                //sample only      
                vm.paintMatrix(4); 
		vg.paintGraph(4);  
		info.setInfo(info.DEMO_4);
		ScreenNumber = 4;
                return true;       
            }                      
            if (arg.equals("5")) { 
                //sample only      
                vm.paintMatrix(5); 
		vg.paintGraph(5);  
		info.setInfo(info.DEMO_5);
		ScreenNumber = 5;
                return true;       
            }                      
            if (arg.equals("6")) { 
                //sample only      
                vm.paintMatrix(6); 
		vg.paintGraph(6);  
		info.setInfo(info.DEMO_6);
		ScreenNumber = 6;
                return true;       
            }                      
	    // *** end of section modified by JL ***
            if (arg.equals("Hide Info")) {
                info.hide();
                return true;
            }
        }
        return false;
    }
   
    public void destroy() {
        Debug.noteln("About to terminate MainFrame");
        System.exit(0);
        // will not return
        Debug.noteln("MainFrame.destroy(): Will not reach here");
    }

    /* method startAction is to be called by the KQML watcher */
    /* it should make appropriate calls to the ViewGraph and  */
    /* ViewMatrix code to update the status of actions in the */
    /* process graph and the COA matrix                       */

    public void startAction(String action, int coa) {
      Debug.noteln("startAction(" + action + ", " + coa + ") called");
      Debug.noteln("StartCOA2 = " + StartCOA2);
      Debug.noteln("StartCOA3 = " + StartCOA3);
      Debug.noteln("ScreenNumber = " + ScreenNumber);
      if (ScreenNumber < 3) {
	vm.paintMatrix(3);
	info.setInfo(info.DEMO_3);
      }
      if (ScreenNumber > 3) {
	Debug.noteln("KQML event ignored");
      }
      else {
	if (coa == 2) {
	  StartCOA2 = true;
	  if (StartCOA3 == true) {
	    vm.paintMatrix(31);
	    vm.paintMatrix(32);
	    vm.paintMatrix(33);
	    vg.paintGraph(33);
	  }
	  if (StartCOA3 == false) {
	    vm.paintMatrix(31);
	    vg.paintGraph(31);
	  }
	}
	if (coa == 3) {
	  StartCOA3 = true;
	  if (StartCOA2 == true) {
	    vm.paintMatrix(31);
	    vm.paintMatrix(32);
	    vm.paintMatrix(33);
	    vg.paintGraph(33);
	  }
	  if (StartCOA2 == false) {
	    vm.paintMatrix(32);
	    vg.paintGraph(32);
	  }
	}
      }
    }

}
