/** File: ViewGraph.java
*** Contains: Visual class that the process elements sit on in CPE.
*** Author: Steve Polyak <Steve_Polyak@ed.ac.uk>
*** Created: June 1997
*** Updated: Wed Jan 21 19:14:38 1998 by Jeff Dalton
*/

package oplan.acp3;

import java.util.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.applet.Applet;
import oplan.util.*;
// import CPEProcess;
// import MainFrame;

class ViewGraph extends Panel implements Runnable {
  
  MainFrame m_frame;
  CPEProcess m_process;
  Thread m_thread;
  
  private Dimension myDimension;

  final static String NEW =    "New";
  final static String OPEN =   "Open";
  final static String SAVE =   "Save";
  final static String EXIT =   "Exit";
  final static String CONTENTS =   "Contents";
  final static String ABOUT =   "About";
  final static String ADDACT =   "Add Activity";
  final static String ADDDUM =   "Add Dummy";
  final static String DELSEL =   "Delete Selection";
  final static String LOAD1 =   "Load State 1";
  final static String LOAD2 =   "Load State 2";
  final static String LOAD3 =   "Load State 3";
  final static String LOAD4 =   "Load State 4";
  final static String STEP =   "Step";

  ViewGraph(MainFrame frame) {
    m_frame = frame;
    m_process = new CPEProcess(this);
    this.setBackground(Color.white);
    myDimension = new Dimension(582,176);
  }
  
  public void run() {
    while (true) {
      repaint();
      try {
	Thread.sleep(200);
      } catch (InterruptedException e) {
	break;
      }
    }
  }
  
  public synchronized void update(Graphics g) {
    m_process.update(g);
  }
  
  public synchronized boolean mouseDown(Event evt, int x, int y) {
    return m_process.mouseDown(evt, x, y);
  }
  
  public synchronized boolean mouseDrag(Event evt, int x, int y) {
    return m_process.mouseDrag(evt, x, y);
  }
  
  public synchronized boolean mouseUp(Event evt, int x, int y) {
    return m_process.mouseUp(evt, x, y);
  }
  
  public synchronized boolean mouseMove(Event evt, int x, int y) {
    return m_process.mouseMove(evt, x, y);
  }

  public Dimension minimumSize() {
    return myDimension;
  }
  
  public Dimension preferredSize() {
    return myDimension;
  }

  public void start() {
    m_thread = new Thread(this);
    m_thread.start();
    m_thread.setPriority(Thread.NORM_PRIORITY);
    m_process.newProcess();
    m_process.clear();
    repaint();
  }

  public void stop() {
    m_thread.stop();
  }
  public void addActivity(){
    m_process.addActivity();
  }
  public void paintGraph(int n) {   // added by JL
    String file = "";
    String thisLine;
    m_process.clear();
    switch (n)
      {
      case 0:
	file = "cpe00.txt";
	break;
      case 1:
	file = "cpe01.txt";
	break;
      case 2:
	file = "cpe02.txt";
	break;
      case 3:
	file = "cpe03.txt";
	break;
      case 31:
	file = "cpe031.txt";
	break;
      case 32:
	file = "cpe032.txt";
	break;
      case 33:
	file = "cpe033.txt";
	break;
      case 4:
	file = "cpe04.txt";
	break;
      case 5:
	file = "cpe05.txt";
	break;
      case 6:
	file = "cpe06.txt";
	break;
      }
    try {
      FileInputStream fin = new FileInputStream("ProcessModels/" + file);
      DataInputStream myInput = new DataInputStream(fin);
      while((thisLine = myInput.readLine()) != null) {
	parseAndLoad(thisLine);
      }
    }
    catch(IOException e) {
      System.err.println("Error: " + e);
    }
    m_process.locateNode("develop JFACC guidance").setExpanded(true,"");
    m_process.locateNode("develop COA").setExpanded(true,"");
    m_process.locateNode("brief on COA(s)").setExpanded(true,"");
    repaint();
  }
    
  public void MenuNotify(String label) {
    
    if(label.equals(NEW)) {
      m_process.newProcess();
    }
    if(label.equals(OPEN)) {
      fileOpen();
    }
    if(label.equals(STEP)) {
      m_process.locateNode("Act1").setStatus(2);
    }
    if(label.equals(LOAD1)) {
      m_process.clear();
      String thisLine;
      try {
	FileInputStream fin = new FileInputStream("ProcessModels/cpe00.txt");
	DataInputStream myInput = new DataInputStream(fin);
	while ((thisLine = myInput.readLine()) != null) {
	  //Debug.noteln(thisLine);
	  parseAndLoad(thisLine);
	}
      }
      catch (IOException e) {
	System.err.println("Error: " + e);
      }	
    }
    if(label.equals(LOAD2)) {
      m_process.clear();
      String thisLine;
      try {
	FileInputStream fin = new FileInputStream("ProcessModels/cpe01.txt");
	DataInputStream myInput = new DataInputStream(fin);
	while ((thisLine = myInput.readLine()) != null) {
	  //Debug.noteln(thisLine);
	  parseAndLoad(thisLine);
	}
      }
      catch (IOException e) {
	System.err.println("Error: " + e);
      }	
    }
    if(label.equals(LOAD3)) {
      m_process.clear();
      String thisLine;
      try {
	FileInputStream fin = new FileInputStream("ProcessModels/cpe02.txt");
	DataInputStream myInput = new DataInputStream(fin);
	while ((thisLine = myInput.readLine()) != null) {
	  //Debug.noteln(thisLine);
	  parseAndLoad(thisLine);
	}
      }
      catch (IOException e) {
	System.err.println("Error: " + e);
      }	
    }
    if(label.equals(LOAD4)) {
      m_process.clear();
      String thisLine;
      try {
	FileInputStream fin = new FileInputStream("ProcessModels/cpe03.txt");
	DataInputStream myInput = new DataInputStream(fin);
	while ((thisLine = myInput.readLine()) != null) {
	  //Debug.noteln(thisLine);
	  parseAndLoad(thisLine);
	}
      }
      catch (IOException e) {
	System.err.println("Error: " + e);
      }	
    }
    if(label.equals(SAVE)) {
      fileSave();
    }
    if(label.equals(ADDACT)) {
      m_process.addActivity();
    }
    if(label.equals(ADDDUM)) {
      m_process.addDummy();
    }
    if(label.equals(DELSEL)) {
      m_process.removeSelected();
    }
    if(label.equals("Delete")) {
      if(m_frame.m_hitNode != null) {
        m_process.removeNode(m_frame.m_hitNode);
      }
    }
  }

  private void fileOpen() {
    Debug.noteln("ViewGraph.fileOpen() called");
  }
  
  private void fileSave() {
    Debug.noteln("ViewGraph.fileSave() called");
  }

  private void parseAndLoad (String in) {
    switch (in.charAt(0))
      {
      case '%':
	break;
      case 'F':
        StringTokenizer st = new StringTokenizer(in, " ");
	Integer tempX = null;
	Integer tempY = null;
	Integer tempWidth = null;
	Integer tempHeight = null;
	int i = 0;
	for  (Enumeration  e  = st; e.hasMoreElements()  ; i++)  {
	  String temp = (String) e.nextElement();
	  switch (i) 
	    {
	    case 1:
	      tempX = new Integer(temp);
	      break;
	    case 2:
	      tempY = new Integer(temp);
	      break;
	    case 3:
	      tempWidth = new Integer(temp);
	      break;
	    case 4:
	      tempHeight = new Integer(temp);
	      break;
	    }
	}
	//m_frame.move(tempX.intValue(), tempY.intValue());
	//m_frame.resize(tempWidth.intValue(),tempHeight.intValue());
	break;
      case 'N':
	String tmpKey = null;
	Integer tmpX = null;
	Integer tmpY = null;
	String tmpType = null;
	Integer tmpStatus = null;
	String tmpSelected = null;
	String tmpLabel = null;
	StringTokenizer st1 = new StringTokenizer(in, "{}");
	int i1 = 0;
	for  (Enumeration  e  = st1; e.hasMoreElements()  ; i1++)  {	
	  String temp = (String) e.nextElement();
	  switch (i1) {
	  case 1:
	    tmpKey = temp;
	    tmpLabel = temp;
	    break;
	  case 2:
	    StringTokenizer st2 = new StringTokenizer(temp, " ");
	    int j = 0;
	    for  (Enumeration  e2  = st2;
		  e2.hasMoreElements()  ; j++)  {	
	      String temp2 = (String) e2.nextElement();
	      switch (j) {
	      case 0:
		tmpX = new Integer(temp2);
		break;
	      case 1:
		tmpY = new Integer(temp2);
		break;
	      case 2:
		tmpType = temp2;
		break;
	      case 3:
		tmpStatus = new Integer(temp2);
		break;
	      case 4:
		tmpSelected = temp2;
		break;
	      }
	    }
	    break;
	  case 3:
	    tmpLabel = temp;
	    break;
	  }
	}
	//Debug.noteln("tt: " + tmpLabel + tmpX + tmpY +
	//tmpType + tmpStatus + tmpSelected);
	Node tmpNode = null;
	if (tmpType.equals("S")) {
	  tmpNode = new SpecialNode(tmpLabel,tmpX.intValue(),
				    tmpY.intValue());
	}
	else {
          tmpNode = new Node(tmpLabel,tmpX.intValue(),
			     tmpY.intValue());
	}
	tmpNode.setStatus(tmpStatus.intValue());
	tmpNode.m_selected = ((tmpSelected.equals("Y") ? true : false));
	m_process.addNode(tmpNode);
	break;
      case 'E':
	String tmpFrom = null;
	String tmpTo = null;
	StringTokenizer st3 = new StringTokenizer(in, "{}");
	int i3 = 0;
	for  (Enumeration  e  = st3; e.hasMoreElements()  ; i3++)  {	
	  String temp = (String) e.nextElement();
	  switch (i3) 
	    {
	    case 1:
              tmpFrom = temp;
	      break;
	    case 3:
	      tmpTo = temp;
	      break;
	    }
	}
	m_process.addEdge(tmpFrom,tmpTo);      
	break;
      }
  }
}
