/* File: ReadKqmlLog.java
 * Contains: Main program for looking at Kqml log files
 * Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Created: January 1998
 * Updated: Fri Jan 23 16:48:35 1998 by Jeff Dalton
 * Copyright: (c) 1998, AIAI, University of Edinburgh
 */

package oplan.test;

import java.util.*;
import java.io.*;
import oplan.util.*;
import oplan.util.lisp.*;
import oplan.util.match.*;


class ReadKqmlLog {

    public static void main(String[] argv) {

	// If there's a command-line arg, look at that file.

	if (argv.length == 1) {
	    scanLog(argv[0]);
	    System.exit(0);
	}

    }

    static void scanLog(String filename) {
	LispReader lin;
	Object c;		// contents

	System.out.println("Target = " + target);

	try {
	    lin = new LispFileReader(filename);
	    while ((c = safeRead(lin, Lisp.NIL)) != Lisp.EOF) {
		analyse(c);
	    }
	    System.exit(0);
	}
	catch (FileNotFoundException e) {
	    System.out.println(e);
	    System.exit(1);
	}

    }

    static Object safeRead(LispReader lin, Object errValue) {
	try { return lin.readObject(); }
	catch (RuntimeException e) { 	// s.b. LispReadException
	    Debug.noteException(e);
	    return errValue;
	}
    }

    // We look for (evaluate ... (:content <target>) ...)

    static Object target =
      Lisp.readFromString(
       "(:INIT-PROBLEM :TASK \"AS-A\" :PLAN ?plan-advice)");

    static final Symbol S_EVALUATE     = Symbol.intern("evaluate");
    static final Symbol K_CONTENT      = Symbol.intern(":content");
    static final Symbol K_INIT_PROBLEM = Symbol.intern(":init-problem");

    static void analyse(Object c) {
	if (c instanceof Cons && ((Cons)c).car() == S_EVALUATE) {
	    List message = (List)c;
	    List plist = message.cdr();
	    Object content = plist.get(K_CONTENT);
	    MatchEnv env = SimpleMatcher.match(target, content);
	    if (env != null) {
		System.out.println("Found: " + c);
		System.out.println("Env = " + env);
		Util.askLine("ok?");
	    }
	}
    }

}
