/* File: MatchTest.java
 * Contains: Main program for testing Kqml message sending
 * Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Created: January 1998
 * Updated: Fri Jan 23 20:03:15 1998 by Jeff Dalton
 * Copyright: (c) 1998, AIAI, University of Edinburgh
 */

package oplan.test;

import java.util.*;
import java.io.*;
import oplan.util.*;
import oplan.util.lisp.*;

import kqml.*;


/** A class to test Kqml message sending. */

class SendKqml {

    private static String serverName = "ACP3";

    private static KqmlAgent agent = null;

    private static LispReader lispin = new LispReader(System.in);

    static final Symbol S_BYE = Symbol.intern("bye");

    public static void main(String[] argv) {

	// Create the agent.
        try { agent = new KqmlAgent("SendKqml"); }
        catch (kqml.KqmlAgentRegistrationFailure e) { fatalException(e); }
        catch (kqml.KqmlAgentInitFailure e)         { fatalException(e); }

	// Loop asking the user for a message and sending it
	
	System.out.println("Type \"bye\" to exit.");

    askloop:
	for (;;) {
	    try {
		Object input = ask("Message:");
		if (input == S_BYE)
		    break askloop;
		else if (input instanceof Cons)
		    sendMessageFromList((Cons)input);
		else
		    System.out.println("I don't understand " + input);
	    }
	    catch (KqmlException e) {
		fatalException(e);
	    }
	    catch (RuntimeException e) {  // s.b. LispReadException
		Debug.noteException(e);
	    }
	}

	// And we're done.
	agent.shutdown();	// needed?

    }

    private static void fatalException(Exception e) {
        System.out.println(e.getMessage());
	if (agent != null)
	    agent.shutdown();	// needed?
	System.exit(1);
    }

    static Object ask(String prompt) {
	System.out.print(prompt + " ");
	System.out.flush();
	return lispin.readObject();
    }

    // A message might be:
    //
    // (EVALUATE :SENDER "pcm-1" :REPLY-WITH :INIT-PROBLEM :CONTENT
    //   (:INIT-PROBLEM :TASK "AS-A" :PLAN "plan-advice1"))

    private static void sendMessageFromList(List message) 
	 throws KqmlException
    {
	System.out.println(message);
	String performative = message.car().toString();
	String contents = message.cdr().car().toString();
	agent.sendMessage(performative, contents, serverName);
    }


}
