/* File: WatchKqml.java
 * Contains: Main program for testing Kqml message reception
 * Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Created: January 1998
 * Updated: Fri Jan 23 20:09:49 1998 by Jeff Dalton
 * Copyright: (c) 1998, AIAI, University of Edinburgh
 */

package oplan.test;

import java.util.*;
import oplan.util.*;
import java.io.*;

import kqml.*;


/** A class to test Kqml message receiving. */

class WatchKqml {

    private static KqmlAgent agent = null;

    private static String serverName = "ACP3";

    private static final long SLEEP_TIME  = 1000;

    public static void main(String[] argv) {

	// Create the agent.
	try {
	    agent = new KqmlAgent(serverName);
	}
        catch (kqml.KqmlAgentRegistrationFailure e) { printException(e); }
        catch (kqml.KqmlAgentInitFailure e)         { printException(e); }

	// Register hendler(s).
	agent.registerHandler("foo", new Watch("bar", agent));

	// Do nothing but sleep, forever.
	// Why even sleep?  Can't this thread just end?  /\/
	for (;;) {
	    try { Thread.sleep(SLEEP_TIME); }
	    catch(InterruptedException e) { printException(e); }
	}

    }

    private static void printException(Exception e) {
        System.out.println(e.getMessage());
	if (agent != null)
	    agent.shutdown();	// needed?
	System.exit(1);
    }

}

class Watch implements Handler {

    String pattern;
    KqmlAgent agent;

    Watch(String pattern, KqmlAgent agent) {
	this.pattern = pattern;
	this.agent = agent;
    }

    public Reply handler(Message msg) {
	System.out.println("Received: " +
			   msg.getPerformative() +
			   ", content = " +
			   msg.getContents());
	if (msg.getContents().equals("die")) {
	    agent.shutdown();
	    System.exit(0);
	}
	return null;
    }

}
