package oplan.util.lisp;

import java.util.*;
import oplan.util.*;


/** Non-empty lists. */

public class Cons extends List {

    Object car;
    List cdr;			// only proper lists

    public Cons(Object car, List cdr) {
	this.car = car;
	this.cdr = cdr;
    }


    public boolean isNull() { return false; }

    public Object car() { return car; }
    public List cdr() { return cdr; }

    public int length() {
	int i = 1;
	for (Cons at = this; at.cdr != Lisp.NIL; at = (Cons)at.cdr, ++i)
	    {}
	return i;
    }

    public Object elementAt(int i) {
	int j = 0;
	for (List at = this; at != Lisp.NIL; at = ((Cons)at).cdr) {
	    if (j++ == i)
		return ((Cons)at).car;
	}
	return Lisp.NIL;
    }

    public Enumeration elements() {
	return new ConsEnumeration(this);
    }

    public boolean equal(List list) {
	return Lisp.equal(this, list);
    }

    public boolean find(Object a) {
	for (List at = this; at != Lisp.NIL; at = ((Cons)at).cdr) {
	    if (((Cons)at).car == a)
		return true;
	}
	return false;
    }

    public List append(List tail) {
	return new Cons(this.car, this.cdr.append(tail));
    }

    public String toString() {
	String s = "(" + this.car.toString();
	Cons at;
	for (at = this; at.cdr instanceof Cons; at = (Cons)at.cdr) {
	    s += " " + ((Cons)at.cdr).car.toString();
	}
	Debug.assert(at.cdr == Lisp.NIL);
	return s + ")";
    }

}


/** Cons enumerations. */

class ConsEnumeration extends ListEnumeration implements Enumeration {

    protected List tail;

    ConsEnumeration(Cons c) {
	tail = c;
    }

    public boolean hasMoreElements() {
	return tail instanceof Cons;
    }

    public Object nextElement() {
	Object elt = ((Cons)tail).car;
	tail = ((Cons)tail).cdr;
	return elt;
    }

}
