/* File: MatchCase.java
 * Contains: A class for pattern-matching
 * Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Created: January 1998
 * Updated: Mon Jan 19 22:57:30 1998 by Jeff Dalton
 * Copyright: (c) 1998, AIAI, University of Edinburgh
 */

package oplan.util.match;

import java.util.*;

/** The root class for entries in MatchTables.

  If you think of a MatchTable as analogous to a case statement, 
  a MatchCase is analogous to a single case clause, with the
  MatchCase's selected method containing the code that is executed
  in the body of that clause.

  @see MatchTable
  @see MatchEnv */

public abstract class MatchCase {

    /** An object to match against. */

    public Object pattern;

    /** tryMatch attempts to match against the pattern, returning
      null if the match fails and some non-null object (such as a 
      MatchEnv) if the match succeeds. */

    public abstract Object tryMatch(Object data);

    /** selected is called by the MatchTable when tryMatch returns
      a non-null result.  That result is passed as the argument to
      selected. */

    public abstract Object selected(Object matchResult);

}
