/* File: SimpleMatcher.java
 * Contains: A simple pattern-matcher
 * Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Created: January 1998
 * Updated: Tue Jan 20 21:36:15 1998 by Jeff Dalton
 * Copyright: (c) 1998, AIAI, University of Edinburgh
 */

package oplan.util.match;

import java.util.*;
import oplan.util.*;
import oplan.util.lisp.*;


public class SimpleMatcher {

    // match returns an env or null.
    // only the pattern can contain variables.
    // if a var appears twice, there's no attempt to check that it
    //   matches the same thing each time.
    // emptyEnv is used to return non-null without allocating an env.

    private static final MatchEnv emptyEnv = new MatchEnv();

    public static MatchEnv match(Object pat, Object dat) {
	return match(pat, dat, null);
    }

    public static MatchEnv match(Object pat, Object dat, MatchEnv env) {
	if (pat instanceof ItemVar) {
	    if (env == null)
		env = new MatchEnv();
	    Debug.assert(env != emptyEnv); // don't modify emptyEnv!
	    env.put(pat, dat);
	    return success(env);
	}
	else if (pat == dat) {
	    return success(env);
	}
	else if (pat.getClass() != dat.getClass()) {
	    return null;
	}
	else if (pat instanceof Symbol) {
	    return null;	// they should have been ==
	}
	else if (pat instanceof String) {
	    return when(((String)pat).equals((String)dat), env);
	}
	else if (pat instanceof Cons) {
	    Cons p = (Cons)pat;
	    Cons d = (Cons)dat;
	    MatchEnv e = match(p.car(), d.car(), env);
	    if (e == null)
		return null;
	    else if (e == emptyEnv) {
		return match(p.cdr(), d.cdr(), null);
	    }
	    else {
		return when(match(p.cdr(), d.cdr(), e) != null, e);
	    }
	}
	else {
	    return null;
	}

    }

    private static MatchEnv when(boolean cond, MatchEnv env) {
	if (cond)
	    return success(env);
	else
	    return null;
    }

    private static MatchEnv success(MatchEnv env) {
	if (env == null)
	    return emptyEnv;
	else
	    return env;
    }

}
